const LOG = require("npm-niosz-log");
const log = LOG.newLog("oracle");
var json = require("npm-niosz-json");
const oracledb = require("oracledb");
if (process.env.LD_LIBRARY_PATH) oracledb.initOracleClient({libDir: process.env.LD_LIBRARY_PATH});
const defaultConfigSize = 5000;
const defaultFetchSize = 2000;
var uuid = require("uuid");
var fs = require("fs");
// ****************************************************************************
var checkSelectConfig = function(obj) {
    var resp = {};
    if (typeof obj === 'object' && obj != null) {
        for (prop in obj) 
            resp[prop] = obj[prop];
        if (!resp.configSize) resp.configSize = defaultConfigSize;
        if (!resp.fetchSize) resp.fetchSize = defaultFetchSize;
        if (resp.fetchSize > resp.configSize) resp.fetchSize = resp.configSize;
    }
    return resp;
};
// ****************************************************************************
var generateConnectionConfig = function(obj) {
    var resp = {
        user : null,
        password : null,
        connectString : null
    };
    if (typeof obj === 'object' && obj != null) {
        if (obj.user) resp.user = obj.user;
        if (obj.password) resp.password = obj.password;
        if (obj.connectString) resp.connectString = obj.connectString;
    }
    return resp;
};
// ****************************************************************************
var generateExcuteConfig = function(obj) {
    var resp = {
        outFormat:oracledb.OUT_FORMAT_OBJECT,
        prefetchRows:defaultConfigSize,
        fetchArraySize:defaultConfigSize,
        resultSet:true
    };
    if (typeof obj === 'object' && obj != null) {
        if (obj.configSize) {
            resp.prefetchRows = obj.configSize;
            resp.fetchArraySize = obj.configSize;
        }
    }
    return resp;
};
// ****************************************************************************
var generateFileName = function(i) {
    return i.toString().padStart(7,"0")+".json";
};
// ****************************************************************************
var oraCamelCase = function(column) {
    column = column.toLowerCase();
    column = column.split("_");
    for(var i=1;i<column.length;i++) {
        if (column[i].length>1) {
            column[i] = column[i][0].toUpperCase() + column[i].substring(1);
        } else {
            column[i] = column[i].toUpperCase();
        }
    }
    return column.join("");
};
// ****************************************************************************
var oraConnect = async function(connectionConfig) {
    try {
        var oracle = await oracledb.getConnection(connectionConfig);
        log.i("Connesso correttamente al database oracle");
        return oracle;
    } catch(e) {
        log.e("Connessione in errore : " + e);
        return null;
    }
};
// ****************************************************************************
var oraDisconnect = async function(connection,resultSet) {
    try {
        if (resultSet) try { await resultSet.resultSet.close(); } catch {};
        await connection.release();
        log.i("Disconnesso correttamente dal database Oracle");
    } catch(e) {
        log.w("Disconnesso con segnalazioni dal database Oracle:\n"+e);
    }
};
// ****************************************************************************
var oraSelect = async function(selectConfig,queryConfig) {

    // *** check select config
    const selectionConfig = checkSelectConfig(selectConfig);

    // *** generate connection config
    const connectionConfig = generateConnectionConfig(selectionConfig);

    // *** connection to oracle
    var oracle = await oraConnect(connectionConfig);
    if (oracle == null) return null;

    // *** generate execute config
    var executeConfig = generateExcuteConfig(selectionConfig);

    // *** esecuzione query
    var queryData = null;
    try {
        queryData = await oracle.execute(queryConfig,[],executeConfig);
        log.i("Query eseguita correttamente");
    } catch(e) {
        log.e("Errore nell'esecuzione della query : " + e);
        return null;
    }

    // *** prepare folder
    var folderData = null;
    try {
        folderData = "oradata/" + uuid.v4();
        fs.mkdirSync(folderData,{recursive:true});
        log.i("Creata cartella dati : " + folderData);
    } catch(e) {
        log.e("Errore nella creazione cartella dati : " + e);
        return null;
    }

    // *** fetching data
    var fetch = 0;
    var len = 0;
    var eof = false;
    while (!eof) {
        var rows = null;
        try {
            rows = await queryData.resultSet.getRows(selectionConfig.configSize);
        } catch(e) {
            log.e("Errore durante il fetching dei dati : " + e);
            await oraDisconnect(oracle,queryData);
            try{fs.rmSync(folderData,{recursive:true,force:true});}catch{};
            return null;
        }
        eof = (!rows.length) ? true : false;
        if (eof) continue;
        try {
            if (selectionConfig.configSize == selectionConfig.fetchSize) {
                json.stringifyToFile(folderData+"/"+generateFileName(fetch),rows);
                fetch++;
                len += rows.length;
            } else {
                var block = [];
                while (rows.length > 0) {
                    block.push(rows.shift());
                    if (block.length == selectionConfig.fetchSize) {
                        json.stringifyToFile(folderData+"/"+generateFileName(fetch),block);
                        fetch++;
                        len += block.length;
                        block = [];
                    }
                }
                if (block.length > 0) {
                    json.stringifyToFile(folderData+"/"+generateFileName(fetch),block);
                    fetch++;
                    len += block.length;
                    block = [];
                }
            }
        } catch(e) {
            log.e("Errore durante la scrittura fetching dati : " + e);
            await oraDisconnect(oracle,queryData);
            try{fs.rmSync(folderData,{recursive:true,force:true});}catch{};
            return null;
        }
        log.i("Record processati : " + len);
    }
    log.i("Record totali processati : " + len);

    // *** disconnection from oracle
    await oraDisconnect(oracle,queryData);
    
    // *** create management resultset response
    return {
      folder : folderData,
      fetchs : fetch,
      rows : len  
    };

};
// ****************************************************************************
var oraResultSet = function(folder,remove) {
    return json.parseFromDirFetch(folder,remove);
}
// ****************************************************************************
const ora = {
    select : oraSelect,
    resultSet : oraResultSet
};
// ****************************************************************************
module.exports = ora;